// Transform
uniform mat4 mv;
uniform mat4 nmv;
uniform mat4 mvp;

// Light
uniform vec4 lightPos0;
uniform vec4 lightColor0;
uniform float lightAtt0;

uniform vec4 lightPos1;
uniform vec4 lightColor1;
uniform float lightAtt1;

uniform vec4 lightPos2;
uniform vec4 lightColor2;
uniform float lightAtt2;

uniform vec4 ambientColor;

uniform vec4 fogColor;
uniform vec2 fogRange;

uniform vec4 materialSpec;
uniform vec4 materialEmis;

// Vertex attr
attribute vec4 pos;
attribute vec4 norm;
attribute vec4 color;

// Output
varying vec4 outColor;
varying vec4 outFogColor;

void main()
{
	gl_Position = mvp * pos;
	
	vec3 wnorm = normalize((nmv * norm).xyz);
	vec4 wpos = mv * pos;
	vec3 dir;
	float dist, att;
	vec4 diffuse;

	// Ambient
	diffuse = ambientColor;

	// Light 0
	dir = lightPos0.xyz - wpos.xyz;
	dist = length(dir);
	dir = normalize(dir);
	att = 1.0 - clamp(dist/lightAtt0-0.5, 0.0, 1.0);
	diffuse += max(0.0, dot(wnorm, dir)) * att * lightColor0;

	// Light 1
	dir = lightPos1.xyz - wpos.xyz;
	dist = length(dir);
	dir = normalize(dir);
	att = 1.0 - clamp(dist/lightAtt1-0.5, 0.0, 1.0);
	diffuse += max(0.0,dot(wnorm, dir)) * att * lightColor1;

	// Light 2
	dir = lightPos2.xyz - wpos.xyz;
	dist = length(dir);
	dir = normalize(dir);
	att = 1.0 - clamp(dist/lightAtt2-0.5, 0.0, 1.0);
	diffuse += max(0.0,dot(wnorm, dir)) * att * lightColor2;
	
	outFogColor = vec4(fogColor.xyz, clamp((length(wpos)-fogRange.x)/fogRange.y,0.0,1.0));
	
	outColor = diffuse*color + materialEmis;
	outColor.w = color.w;
}
